<?php // phpcs:ignore

namespace WP2FA\Shortcodes;

use \WP2FA\WP2FA as WP2FA;
use \WP2FA\Admin\UserNotices as UserNotices;
use \WP2FA\Authenticator\Authentication as Authentication;
use \WP2FA\Core as Core;
use \WP2FA\Authenticator\BackupCodes as BackupCodes;
use \WP2FA\Admin\UserProfile as UserProfile;

/**
 * Class for handling our crons.
 */
class Shortcodes {

	/**
	 * Constructor.
	 */
	public function __construct() {
		add_shortcode( 'wp-2fa-setup-form', array( $this, 'user_setup_2fa_form' ) );
		add_shortcode( 'wp-2fa-setup-notice', array( $this, 'user_setup_2fa_notice' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'register_2fa_shortcode_scripts' ) );
	}

	/**
	 * Register scripts and styles.
	 */
	public function register_2fa_shortcode_scripts() {
		// Add our front end stuff, which we only want to load when the shortcode is present.
		wp_register_script( 'wp_2fa_frontend_scripts', Core\script_url( 'wp-2fa', 'admin' ), array( 'jquery' ), WP_2FA_VERSION, true );
		wp_register_script( 'wp_2fa_micro_modals', Core\script_url( 'micro-modal', 'admin' ), array(), WP_2FA_VERSION, true );
		wp_register_style( 'wp_2fa_styles', Core\style_url( 'styles', 'frontend' ) );
	}

	/**
	 * Output setup form.
	 */
	public function user_setup_2fa_form( $atts ) {
		extract(
			shortcode_atts(
				array(
					'show_preamble' => 'true',
				),
				$atts
			)
		);

		if ( ! is_admin() && is_user_logged_in() ) {
			wp_enqueue_script( 'wp_2fa_frontend_scripts' );
			wp_enqueue_script( 'wp_2fa_micro_modals' );
			wp_enqueue_style( 'wp_2fa_styles' );

			$data_array = array(
				'ajaxURL'        => admin_url( 'admin-ajax.php' ),
				'roles'          => WP2FA::wp_2fa_get_roles(),
				'nonce'          => wp_create_nonce( 'wp-2fa-settings-nonce' ),
				'codesPreamble'  => esc_html__( 'These are the 2FA backup codes for the user', 'wp-2fa' ),
				'readyText'      => esc_html__( 'I\'m ready', 'wp-2fa' ),
				'codeReSentText' => esc_html__( 'New code sent', 'wp-2fa' ),
				'allDoneHeading' => esc_html__( 'All done.', 'wp-2fa' ),
				'allDoneText'    => esc_html__( 'Your login just got more secure.', 'wp-2fa' ),
				'closeWizard'    => esc_html__( 'Close Wizard', 'wp-2fa' ),
			);
			wp_localize_script( 'wp_2fa_frontend_scripts', 'wp2faData', $data_array );

			$forms = new UserProfile();
			ob_start();
			echo '<form id="your-profile" class="wp-2fa-configuration-form">';
				$forms->inline_2fa_profile_form( 'output_shortcode', $show_preamble );
			echo '</form>';
			$content = ob_get_contents();
			ob_end_clean();
			return $content;
		} elseif ( ! is_admin() && ! is_user_logged_in() ) {
			if ( ! empty( WP2FA::get_wp2fa_setting( 'custom-user-page-id' ) ) ) {
				$new_page_id = WP2FA::get_wp2fa_setting( 'custom-user-page-id' );
				$redirect_to = get_permalink( $new_page_id );
			}
			ob_start();
			echo '<p>' . esc_html__( 'You must be logged in to view this page.', 'wp-2fa' ) . ' <a href="' . esc_url( wp_login_url( $redirect_to ) ) . '">' . esc_html__( 'Login here.', 'wp-2fa' ) . '</a></p>';
			$content = ob_get_contents();
			ob_end_clean();
			return $content;
		}
	}

	/**
	 * Output setup nag.
	 */
	public function user_setup_2fa_notice( $atts ) {
		extract(
			shortcode_atts(
				array(
					'configure_2fa_url' => '',
				),
				$atts
			)
		);
		$notice = new UserNotices();

		if ( ! is_admin() && is_user_logged_in() ) {
			wp_enqueue_script( 'wp_2fa_frontend_scripts' );
			wp_enqueue_script( 'wp_2fa_micro_modals' );
			wp_enqueue_style( 'wp_2fa_styles' );

			$data_array = array(
				'ajaxURL'        => admin_url( 'admin-ajax.php' ),
				'roles'          => WP2FA::wp_2fa_get_roles(),
				'nonce'          => wp_create_nonce( 'wp-2fa-settings-nonce' ),
				'codesPreamble'  => esc_html__( 'These are the 2FA backup codes for the user', 'wp-2fa' ),
				'readyText'      => esc_html__( 'I\'m ready', 'wp-2fa' ),
				'codeReSentText' => esc_html__( 'New code sent', 'wp-2fa' ),
				'allDoneHeading' => esc_html__( 'All done.', 'wp-2fa' ),
				'allDoneText'    => esc_html__( 'Your login just got more secure.', 'wp-2fa' ),
				'closeWizard'    => esc_html__( 'Close Wizard', 'wp-2fa' ),
			);
			wp_localize_script( 'wp_2fa_frontend_scripts', 'wp2faData', $data_array );

			ob_start();
			echo $notice->user_setup_2fa_nag( 'output_shortcode', $configure_2fa_url );
			$content = ob_get_contents();
			ob_end_clean();
			return $content;
		}
	}

}
